import java.awt.*;
import java.awt.event.*;

public class Calcolatrice extends Frame implements ActionListener
{
  // Bottoni per le 10 cifre e per i comandi + - * / = C
  Button tasti[];

  // Pannello per contenere i bottoni
  private Panel areaTasti;

  // Display centrale
  private TextField risultato;

  // Operazione correntemente invocata
  char operaz;

  // Primo e secondo argomento correnti
  int arg1;
  int arg2;

  // Stato attuale
  static final int HO_NIENTE = 0; // arg1, operaz, arg2 indefiniti
  static final int HO_ARG1 = 1;   // arg1 definito, operaz e arg2 no
  static final int HO_OP = 2;     // arg1 e operaz definiti, arg2 no
  static final int HO_ARG2 = 3;   // arg1, operaz, arg2 definiti
  int stato = -1; // indefinito

  /* Costruttore, crea un frame di tipo calcolatrice, non lo visualizza */
  public Calcolatrice()
  {
    // Alloca e inizializza i bottoni della tastiera.
    tasti = new Button[16];
    tasti[0] = new Button("1");
    tasti[1] = new Button("2");
    tasti[2] = new Button("3");
    tasti[3] = new Button("+");
    tasti[4] = new Button("4");
    tasti[5] = new Button("5");
    tasti[6] = new Button("6");
    tasti[7] = new Button("-");
    tasti[8] = new Button("7");
    tasti[9] = new Button("8");
    tasti[10] = new Button("9");
    tasti[11] = new Button("*");
    tasti[12] = new Button("0");
    tasti[13] = new Button("C");
    tasti[14] = new Button("=");
    tasti[15] = new Button("/");

    // Prepara l'area della tastiera organizzandola come griglia.
    // Aggiunge i bottoni riga per riga a partire dall'alto.
    areaTasti = new Panel();
    areaTasti.setLayout(new GridLayout(4,4)); // 4 righe, 4 colonne
    int i;
    for (i=0; i<16; i++) {  areaTasti.add(tasti[i]);  }

    // Prepara il display.
    risultato = new TextField(10);
    risultato.setEditable(false);

    // Riempe la finestra principale della calcolatrice.
    setLayout( new BorderLayout() );
    add(risultato, BorderLayout.NORTH);
    add(areaTasti, BorderLayout.CENTER);
    setTitle("Cacolatrice");

    // Fa calcolare al sistema le dimensioni.
    pack();

    // Assegna comportamenti ai bottoni.
    for (i=0; i<16; i++)
    {  tasti[i].addActionListener(this);  }

    // Stabilisce lo stato iniziale.
    aggiornaStato(HO_NIENTE);
  }

  /* Main, visualizza la calcolatrice */
  public static void main (String argv[])
  {
    Calcolatrice calc = new Calcolatrice();
    calc.addWindowListener (new WindowAdapter()
    {
      public void windowClosing(WindowEvent e)
       {  System.exit(0); }
    });
    calc.setVisible(true);
  }

  /* Aggiorna l'interfaccia portandola in stato s */
  private void aggiornaStato(int s)
  {
    // Bitmask con le abilitazioni per i tasti:
    // bitmask[i] == true sse in questo stato tasti[i] va abilitato
    int bitmask_niente[] = {1,1,1,0,1,1,1,0,1,1,1,0,1,0,0,0};
    int bitmask_arg1[] =   {1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,1};
    int bitmask_op[] =     {1,1,1,0,1,1,1,0,1,1,1,0,1,1,0,1};
    int bitmask_arg2[] =   {1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1};
    int i;
    if (stato != s)
    {
       stato = s;
       switch (stato)
       {
         case HO_NIENTE:
           eseguiAbilitazioni(bitmask_niente); break;
         case HO_ARG1:
           eseguiAbilitazioni(bitmask_arg1); break;
         case HO_OP:
           eseguiAbilitazioni(bitmask_op); break;
         case HO_ARG2:
           eseguiAbilitazioni(bitmask_arg2); break;
       }
    }
    switch (stato)
    {
      case HO_NIENTE: risultato.setText(""); break;
      case HO_ARG1:
      case HO_OP:     risultato.setText("" + arg1); break;
      case HO_ARG2:   risultato.setText("" + arg2); break;
    }
  }

  /* Abilita e disabilita bottoni in base alla bitmask. Per chiarezza
     colora i bottoni abilitati in verde e i disabilitati in rosso */
  private void eseguiAbilitazioni(int[] bitmask)
  {
    int i;
    for (i=0; i<16; i++)
    {  if (bitmask[i] == 1)
       { tasti[i].setEnabled(true); tasti[i].setBackground(Color.green); }
       else
       { tasti[i].setEnabled(false); tasti[i].setBackground(Color.red); }
    }
  }

  /* Gestisce tasti cifra 0 1 2 3 4 5 6 7 8 9 */
  private void tastoCifra(char c)
  {
    // ritrova il numero rappresentato dal carattere
    int n = (int)(c) - (int)('0');
    switch (stato)
    {  case HO_NIENTE:
         arg1 = n; aggiornaStato(HO_ARG1); break;
       case HO_ARG1:
         arg1 = arg1*10 + n; aggiornaStato(HO_ARG1); break;
       case HO_OP:
         arg2 = n; aggiornaStato(HO_ARG2); break;
       case HO_ARG2:
         arg2 = arg2*10 + n; aggiornaStato(HO_ARG2); break;
    }
  }

  /* Esegue operazione aritmetica corrispondente al carattere dato */
  private void eseguiOperazione(char op)
  {
     {
       switch (op)
       {  case '+': arg1 = arg1+arg2; break;
          case '-': arg1 = arg1-arg2; break;
          case '*': arg1 = arg1*arg2; break;
          case '/': arg1 = arg1/arg2; break;
       }
     }
  }

  /* Gestisce tasti funzione + - * / = C */
  private void tastoFunzione(char op)
  {
    switch (op)
    {  case 'C':
         aggiornaStato(HO_NIENTE); break;
       case '=':
         eseguiOperazione(operaz); aggiornaStato(HO_ARG1); break;
       case '+':
       case '-':
       case '*':
       case '/':
          if (stato==HO_ARG2) eseguiOperazione(operaz);
          operaz = op;
          aggiornaStato(HO_OP);
          break;
    }
  }

   public void actionPerformed (ActionEvent e)
   {
     // Preleva il carattere che etichetta il bottone.
     String s = e.getActionCommand();
     char c = s.charAt(0);
     // Esegue procedura corrispondente.
     if ((c>='0') && (c<='9')) tastoCifra(c);
     else tastoFunzione(c);
   }
}

